! -*- mode: F90 ; mode: font-lock ; column-number-mode: true ; vc-back-end: RCS -*-
!=============================================================================!
!                           M A I N                                           !
!=============================================================================!
!                                                                             !
! $Id: Main.F90,v 1.3 2004/03/11 14:49:42 dq100 Exp $
!                                                                             !
!-----------------------------------------------------------------------------!
! Main program unit. Reads the input file and then calls the analysis programs!
! as required.                                                                !
!                                                                             !
!-----------------------------------------------------------------------------!
! Written by D. Quigley  - University of York                                 !
!-----------------------------------------------------------------------------!
!                                                                             !
! $Log: Main.F90,v $
! Revision 1.3  2004/03/11 14:49:42  dq100
! Added XSF output option
!
! Revision 1.2  2004/02/27 11:51:27  dq100
! Added xmol routine - wrapped co-ords only so far
!
! Revision 1.1.1.1  2004/01/29 16:31:14  dq100
! Initial revision
!
!
!                                                                             !
!=============================================================================!

Program Main

  Use Constants
  Use Global
  Use IO
  Use Analyse
  Implicit None


    ! Variables to handle the command line
    Integer(kind=int32) :: Num_args,iarg
    Integer,  External ::  iargc
    Character(30), Dimension(0:1) :: command_line


    Integer(kind=int32) :: ierr,FileOutputInt,DummyInt

    Integer(kind=int32) :: Start,Final
    Integer(kind=int32) :: VacLength,VacInterval
    Integer(kind=int32) :: MSDLength,MSDInterval

    Logical :: CalcRDF     = .FALSE.
    Logical :: CalcVAC     = .FALSE.
    Logical :: CalcMSD     = .FALSE.
    Logical :: CalcHeatCap = .FALSE.
    Logical :: CalcBetaT   = .FALSE.
    Logical :: CalcAlphaP  = .FALSE.
    Logical :: CalcTempDist= .FALSE.
    Logical :: CalcVolDist = .FALSE.
    Logical :: XmolFile    = .FALSE.
    Logical :: XSFFile     = .FALSE.

    ! "Undocumented" flag for testing with goldilocs
    Logical :: goldilocs = .FALSE. 

    Real(kind=dp) :: Dummy,DeltaT

    Namelist/RunName/SeedName,goldilocs
    Namelist/Positions/Start,Final
    Namelist/CalcFlags/CalcRDF,CalcVAC,CalcMSD,CalcHeatCap,CalcBetaT, &
                       CalcAlphaP,CalcTempDist,CalcVolDist,Xmolfile, &
                       XSFFile
    Namelist/VACParams/VacLength,VacInterval
    Namelist/MSDParams/MSDLength,MSDInterval

    ! ** READ THE COMMAND LINE
  
    ! Check that there is only one argument.
    num_args = iargc()

    If (num_args/=1) Then
       Write(*,*)
       Write(*,*) '                      M D T E P           '
       Write(*,*)
       Write(*,*) '            Usage:   mdtep <input file>   '
       Write(*,*)
       Write(*,*) '           D. Quigley - University of York'
       Write(*,*)
       Stop
    End If

    Do iarg = 1, num_args
       Call getarg(iarg,command_line(iarg))
    End Do
    
    ! Name of the input file read from the command line

    ! Try and open this file
    Open(unit=22,file=trim(Command_line(1)),status='old',iostat=ierr)
    If (ierr/=0) then
       Write(*,*)"Error reading input file "//trim(Command_line(1))
       Write(*,*)"(Does it actually exist?)"
       Stop
    Else
       ! File opened properly - is the first line valid
       Read(22,nml=RunName,iostat=ierr)
       If (ierr/=0) then
          Write(*,*)"Error reading RunName namelist from "//trim(Command_line(1))
          Write(*,*)"Should be of form:"
          Write(*,*)
          Write(*,*)"&RunName"
          Write(*,*)"SeedName = '<castep seedname>'"
          Write(*,*)"/"
          Write(*,*)
          Stop
       End If
    End If

    ! Read the name lists
    Read(22,nml=Positions,iostat=ierr)
    If (ierr/=0) then
       Write(*,*)"Error reading Positions namelist from "//trim(Command_line(1))
       Stop
    End If
    Read(22,nml=CalcFlags,iostat=ierr)
    If (ierr/=0) then
       Write(*,*)"Error reading CalcFlags namelist from "//trim(Command_line(1))
       Stop
    End If
    Read(22,nml=VACParams,iostat=ierr)
    If (ierr/=0) then
       Write(*,*)"Error reading VACParams namelist from "//trim(Command_line(1))
       Stop
    End If
    Read(22,nml=MSDParams,iostat=ierr)
    If (ierr/=0) then
       Write(*,*)"Error reading MSDParams namelist from "//trim(Command_line(1))
       Stop
    End If


    Close(22)

    ! Call the IO module to read in the md file
    
    If (Goldilocs) then
       Open(unit=27,file='Production.out',status='old',iostat=ierr)
       If (ierr/=0) then
          Write(*,*)"Error - Could not open Production.out file"
          Stop
       End If
       Read(27,'(2i15)' )Num_Atoms,DummyInt  ! Number of atoms
       Read(27,'(i15)'  )MD_Steps            ! Number of outputs in file
       Read(27,'(i15)'  )FileOutputInt       ! Interval in timesteps
       Read(27,'(f15.5)')DeltaT              ! Calculation Timestep
       Close(27)
    Else   
       Call IO_ConvertfromMDFile
    End If

    ! Call the appropriate analysis routines.
    If (CalcRDF)  Call Analyse_CalcRDF(Start,Final)
    If (CalcVAC)  Call Analyse_VACFunction(Start,Final,VACLength,VACInterval)
    If (CalcMSD)  Call Analyse_MSD(Start,Final,MSDLength,MSDInterval)
    If (XmolFile) Call IO_WriteXmolFile
    If (XSFFile)  Call IO_WriteXSFFile

    If (Goldilocs) then
 
      ! All routines from here on in expect MD_Steps
       ! to be the number of samples.
       Open(unit=22,file='Sample.dat',status='old',iostat=ierr)
       If (ierr/=0) then
          Write(*,*)"Cannot open Sample.dat!"
          Stop
       End If
       MD_Steps = 0
       Do
          Read(22,*,end=10)dummy,dummy,dummy,dummy
          MD_Steps = MD_Steps + 1
       End Do
       
10     Close(22)

    End if

    If (CalcHeatCap) Call Analyse_HeatCap(Start,Final)
    If (CalcBetaT)   Call Analyse_BetaT(Start,Final)
    If (CalcAlphaP)  Call Analyse_AlphaP(Start,Final)

    If (CalcTempDist) Call Analyse_TempTest(Start,Final)
    If (CalcVolDist)  Call Analyse_VolTest(Start,Final)
 


    ! Delte the temporary files
!    If (Goldilocs==.FALSE.) Call IO_DeleteTempFiles



End Program Main
